package com.gyf.csams.main.model

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.*
import com.gyf.lib.model.ScrollViewModel
import com.gyf.lib.util.*
import com.orhanobut.logger.Logger
import kotlinx.coroutines.launch


/**
 * 数据状态管理
 *
 */
class AssociationManagementViewModel(application: Application) : ScrollViewModel<AssociationVo>(
    application
) {

    init {
        load()
    }

    fun load() {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AssociationApi.ListAll),
                callback = HttpCallback<MutableList<AssociationVo>>(action = "加载社团",
                    onSuccess = { it ->
                        it.body?.let {
                            _data.postValue(it)
                        }
                    },
                    typeToken = object :
                        TypeToken<ApiResponse<MutableList<AssociationVo>>>() {}.type
                ),
                jsonParam = OnlyToken(clientType = ClientType.Background)
            )
        }
    }

    /**
     *
     * @param level
     */
    fun update(
        associationVo: AssociationVo,
        level: AssociationLevel,
        callback: (m: String) -> Unit
    ) {
        Logger.i("社团:${associationVo.name}")
        HttpClient.post(url = Api.buildUrl(AssociationApi.Update),
            callback = HttpCallback<Boolean>(action = "社团等级更新",
                onSuccess = {
                    callback(it.message)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type
            ),
            jsonParam = UpdateAssociationVo(
                associationVo = associationVo.copy(level = level),
                token = TokenManager.getToken(),
                clientType = ClientType.Background
            )
        )
    }
}