package com.gyf.csams.main.model

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.*
import com.gyf.lib.model.ScrollViewModel
import com.gyf.lib.util.*
import kotlinx.coroutines.launch
import java.lang.reflect.Type


abstract class BaseAuditViewModel<T : AuditVo>(application: Application) :
    ScrollViewModel<T>(application = application) {

    abstract val auditApi: UrlPath
    abstract val acceptApi: UrlPath
    abstract val checkApi: UrlPath

    abstract val typeToken: Type

    /**
     * 加载审核记录
     *
     */
    fun load() {
        viewModelScope.launch {
            HttpClient.post(
                Api.buildUrl(auditApi), HttpCallback<MutableList<T>>(
                    action = "加载资料",
                    onSuccess = { it ->
                        it.body?.let {
                            _data.postValue(it)
                        }
                    },
                    typeToken = typeToken
                ),
                jsonParam = OnlyToken(clientType = ClientType.Background)
            )
        }
    }


    /**
     * 受理资料
     *
     * @param callback
     */
    fun accept(auditId: Int, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            HttpClient.post(
                Api.buildUrl(acceptApi),
                HttpCallback<Boolean>(action = "受理资料", onSuccess = {
                    callback(it.message)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type),
                jsonParam = AcceptVo(auditId = auditId, token = TokenManager.getToken())
            )
        }
    }

    /**
     * 审核资料
     *
     * @param auditId
     * @param result
     * @param callback
     */
    fun check(auditId: Int, cause: String, result: Boolean, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            HttpClient.post(
                Api.buildUrl(checkApi),
                HttpCallback<Boolean>(action = "审核资料", onSuccess = {
                    callback(it.message)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type),
                jsonParam = CheckVo(
                    auditId = auditId,
                    result = result,
                    cause = cause,
                    token = TokenManager.getToken()
                )
            )
        }
    }

}