package com.gyf.csams.main.model

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.AllOfficerVo
import com.gyf.csams.module.ApiResponse
import com.gyf.csams.module.ClientType
import com.gyf.csams.module.ManagerDutySumVo
import com.gyf.lib.util.*
import com.orhanobut.logger.Logger
import kotlinx.coroutines.launch


enum class ColumnType {
    Text,
    DropMenu
}


/**
 * 部门干事数据状态管理
 *
 */
class ManagementOfficerModel : ViewModel() {
    private val _data = MutableLiveData<AllOfficerVo>()
    val data: LiveData<AllOfficerVo> = _data

    //部门概况
    private val _simpleData = MutableLiveData<ManagerDutySumVo>()
    val simpleData: LiveData<ManagerDutySumVo> = _simpleData

    init {
        load()
    }


    //TODO 更新职务
    fun updateDuty() {
        Logger.i("更新职务")
    }

    /**
     *加载部门概况
     */
    private fun load() {
        viewModelScope.launch {
            HttpClient.post(url = Api.buildUrl(AccountApi.Load),
                callback = HttpCallback<ManagerDutySumVo>(action = "加载部门概况", onSuccess =
                { it ->
                    it.body?.let {
                        _simpleData.postValue(it)
                    }
                }, typeToken = object : TypeToken<ApiResponse<ManagerDutySumVo>>() {}.type
                ),
                jsonParam = OnlyToken(clientType = ClientType.Background)
            )
        }
    }

    /**
     * 加载部门详情
     *
     */
    fun loadDetail() {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AccountApi.LoadDetail),
                callback = HttpCallback<AllOfficerVo>(action = "加载部门详情", onSuccess = { it ->
                    it.body?.let {
                        _data.postValue(it)
                    }
                }, typeToken = object : TypeToken<ApiResponse<AllOfficerVo>>() {}.type),
                jsonParam = OnlyToken(clientType = ClientType.Background)
            )
        }
    }
}