package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.*
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.main.model.AssociationManagementViewModel
import com.gyf.csams.module.AssociationLevel
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainBoxFrame
import com.gyf.lib.uikit.ScaffoldModel

/**
 * 社团管理
 *
 */
class AssociationManagementActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            Body {
                MainBoxFrame(background = { /*TODO*/ }) {
                    val model: AssociationManagementViewModel = viewModel()
                    val data by model.data.observeAsState()
                    val listState = rememberLazyListState()
                    LazyColumn(modifier = Modifier.fillMaxSize(), state = listState) {
                        data?.forEach {
                            it.apply {
                                item {
                                    var expanded by remember {
                                        mutableStateOf(false)
                                    }
                                    Card(backgroundColor = MaterialTheme.colors.background) {
                                        Column(
                                            modifier = Modifier.padding(10.dp),
                                            verticalArrangement = Arrangement.SpaceBetween
                                        ) {
                                            Row(
                                                modifier = Modifier.fillMaxWidth(),
                                                horizontalArrangement = Arrangement.SpaceBetween
                                            ) {
                                                Text(
                                                    text = "$associationId",
                                                    style = MaterialTheme.typography.h6
                                                )
                                                Text(
                                                    text = name,
                                                    style = MaterialTheme.typography.h4
                                                )
                                                Spacer(modifier = Modifier)
                                            }
                                            Spacer(modifier = Modifier.height(5.dp))
                                            Row(
                                                modifier = Modifier.fillMaxWidth(),
                                                horizontalArrangement = Arrangement.SpaceBetween,
                                                verticalAlignment = Alignment.CenterVertically
                                            ) {
                                                Text(
                                                    text = ""
                                                    /**TODO 用户**/
                                                    ,
                                                    style = MaterialTheme.typography.h5
                                                )
                                                Text(
                                                    text = faculty.desc,
                                                    style = MaterialTheme.typography.h5
                                                )

                                                OutlinedButton(onClick = {
                                                    expanded = true
                                                }) {
                                                    Text(
                                                        text = level?.name ?: "暂无评级",
                                                        style = MaterialTheme.typography.h5
                                                    )
                                                    val scaffoldModel: ScaffoldModel = viewModel()
                                                    DropdownMenu(
                                                        expanded = expanded,
                                                        onDismissRequest = { /*TODO*/ }) {
                                                        AssociationLevel.values().forEach {
                                                            DropdownMenuItem(onClick = {
                                                                model.update(
                                                                    associationVo = this@apply,
                                                                    level = it
                                                                ) {
                                                                    scaffoldModel.update(
                                                                        message = it,
                                                                        actionLabel = "刷新"
                                                                    ) {
                                                                        model.load()
                                                                    }
                                                                }
                                                                expanded = false
                                                            }) {
                                                                Text(text = it.name)
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                            Spacer(modifier = Modifier.height(5.dp))
                                            Card(
                                                modifier = Modifier
                                                    .fillMaxWidth()
                                                    .height(50.dp)
                                            ) {
                                                Text(text = desc)
                                            }
                                        }
                                    }
                                    Spacer(modifier = Modifier.height(10.dp))
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
