package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.runtime.Composable
import androidx.compose.runtime.ExperimentalComposeApi
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.core.view.WindowCompat
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.insets.ExperimentalAnimatedInsets
import com.gyf.csams.R
import com.gyf.csams.main.model.AuditActViewModel
import com.gyf.csams.module.AuditActVo
import com.gyf.csams.uikit.BASE_HEIGHT
import com.gyf.csams.uikit.CheckForm
import com.gyf.csams.uikit.RowItem
import com.gyf.csams.uikit.TestTableImeSimple
import com.gyf.lib.uikit.ImeBody

import com.gyf.lib.util.DateTimeUtil.datetimeFormat
import java.util.*

/**
 * 审批社团活动
 *
 */
class AuditActActivity : ComponentActivity() {
    @ExperimentalMaterialApi
    @ExperimentalAnimatedInsets
    @ExperimentalComposeApi
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        WindowCompat.setDecorFitsSystemWindows(window, false)
        setContent {
            ImeBody {
                val model: AuditActViewModel = viewModel()
                val data by model.data.observeAsState()
                TestTableImeSimple(
                    title = R.string.activity_association
                ) {
                    data?.forEach {
                        item {
                            RegisterForm(vo = it)
                            CheckForm<AuditActVo, AuditActViewModel>(vo = it)
                            Spacer(modifier = Modifier.height(10.dp))
                        }
                    }
                }
            }
        }

    }

    @Composable
    private fun RegisterForm(
        modifier: Modifier = Modifier,
        vo: AuditActVo
    ) {
        Column(modifier = modifier) {
            vo.activityVo.activityTime
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.activity_name,
                value = vo.activityVo.activityName
            )
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.activity_time,
                value = Date(vo.activityVo.activityTime).datetimeFormat()
            )
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT * 1.5F),
                key = R.string.activity_address,
                value = vo.activityVo.activityAddress
            )
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT * 3),
                key = R.string.activity_desc,
                value = vo.activityVo.activityDesc
            )
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.activity_size,
                value = "${vo.activityVo.activitySize}"
            )
        }
    }
}