package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.Image
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.material.MaterialTheme
import androidx.compose.runtime.Composable
import androidx.compose.runtime.ExperimentalComposeApi
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.core.view.WindowCompat
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.coil.rememberCoilPainter
import com.google.accompanist.insets.ExperimentalAnimatedInsets
import com.gyf.csams.R
import com.gyf.csams.main.model.AuditAssociationViewModel
import com.gyf.csams.module.AuditAssociationVo
import com.gyf.csams.uikit.BASE_HEIGHT
import com.gyf.csams.uikit.CheckForm
import com.gyf.csams.uikit.RowItem
import com.gyf.csams.uikit.TestTableImeSimple
import com.gyf.lib.uikit.ImeBody
import com.gyf.lib.util.Api

class AuditAssociationActivity : ComponentActivity() {
    @ExperimentalMaterialApi
    @ExperimentalAnimatedInsets
    @ExperimentalComposeApi
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        WindowCompat.setDecorFitsSystemWindows(window, false)
        setContent {
            ImeBody {
                val model: AuditAssociationViewModel = viewModel()
                val data by model.data.observeAsState()
                TestTableImeSimple(
                    title = R.string.association_reg_title
                ) {
                    data?.forEach {
                        item {
                            RegisterForm(vo = it)
                            CheckForm<AuditAssociationVo, AuditAssociationViewModel>(vo = it)
                            Spacer(modifier = Modifier.height(10.dp))
                        }
                    }
                }
            }
        }
    }


    @Composable
    private fun RegisterForm(
        modifier: Modifier = Modifier, vo: AuditAssociationVo
    ) {
        Column(
            modifier = modifier.border(
                width = 1.dp,
                color = MaterialTheme.colors.onBackground
            )
        ) {
            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.petitioner,
                value = vo.audit.user.name
            )

            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.association_name,
                value = vo.name
            )

            RowItem(
                modifier = Modifier.height(BASE_HEIGHT),
                key = R.string.association_desc,
                value = vo.desc
            )

            RowItem(
                modifier = Modifier.height(BASE_HEIGHT * 3),
                key = R.string.association_logo
            ) {
                //TODO 图片全屏显示
                val painter =
                    rememberCoilPainter(Api.buildUrl(vo.logo))
                Image(painter = painter, contentDescription = null)
            }
        }
    }
}