package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.height
import androidx.compose.material.ExperimentalMaterialApi
import androidx.compose.runtime.Composable
import androidx.compose.runtime.ExperimentalComposeApi
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.insets.ExperimentalAnimatedInsets
import com.gyf.csams.R
import com.gyf.csams.main.model.AuditRenameViewModel
import com.gyf.csams.module.AuditRenameVo
import com.gyf.csams.uikit.CheckForm
import com.gyf.csams.uikit.RowItem
import com.gyf.csams.uikit.TestTable
import com.gyf.csams.uikit.TestTableImeSimple
import com.gyf.lib.uikit.ImeBody

class AuditRenameActivity : ComponentActivity() {

    @ExperimentalMaterialApi
    @ExperimentalAnimatedInsets
    @ExperimentalComposeApi
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            TestTable(clazz = AuditRenameViewModel::class.java, title = R.string.rename_form) {
                RenameForm(vo = it)
            }
            ImeBody {
                val model: AuditRenameViewModel = viewModel()
                val data by model.data.observeAsState()
                TestTableImeSimple(
                    title = R.string.rename_form
                ) {
                    data?.forEach {
                        item {
                            RenameForm(vo = it)
                            CheckForm<AuditRenameVo, AuditRenameViewModel>(vo = it)
                            Spacer(modifier = Modifier.height(10.dp))
                        }
                    }
                }
            }
        }
    }

    @Composable
    private fun RenameForm(
        modifier: Modifier = Modifier,
        vo: AuditRenameVo
    ) {
        Column(modifier = modifier) {
            val baseHeight = 50.dp
            RowItem(
                modifier = Modifier.height(baseHeight),
                key = R.string.petitioner,
                value = vo.audit.user.name
            )
            RowItem(
                modifier = Modifier.height(baseHeight),
                key = R.string.oldname,
                value = vo.associationVo.name
            )
            RowItem(
                modifier = Modifier.height(baseHeight),
                key = R.string.newname,
                value = vo.renameVo.newName
            )
            RowItem(
                modifier = Modifier.height(baseHeight * 3),
                key = R.string.reason_for_application,
                value = vo.renameVo.cause
            )
        }
    }

}