package com.gyf.csams.main.ui

import android.content.Intent
import android.os.Bundle
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.OutlinedButton
import androidx.compose.material.Text
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.R
import com.gyf.csams.account.model.LoginViewModel
import com.gyf.csams.main.model.MenuType
import com.gyf.csams.module.ClientType
import com.gyf.lib.service.BaseActivity
import com.gyf.lib.uikit.*
import com.gyf.lib.util.TokenManager

class MainActivity : BaseActivity() {

    override val clientType: ClientType = ClientType.Background

    private val choosePhoto: ChoosePhoto = ChoosePhoto(activity = this)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            Body {
                val loginViewModel: LoginViewModel = viewModel()
                val scaffoldModel: ScaffoldModel = viewModel()

                MainColumnFrame(background = { }) {
                    val photoLauncher = choosePhoto.photoLauncher()
                    val permissionLauncher =
                        choosePhoto.permissionLauncher(photoLauncher = photoLauncher)
                    val personInfoData by TokenManager.getPersonInfoData().observeAsState()
                    personInfoData?.let {
                        Profile(
                            modifier = Modifier
                                .weight(0.4F)
                                .padding(10.dp),
                            personInfoVo = it,
                            setHeadImg = {
                                choosePhoto.requestPhoto(
                                    photoLauncher = photoLauncher,
                                    permissionLauncher = permissionLauncher
                                )
                            }
                        )
                    }
                    Column(
                        modifier = Modifier.weight(0.6F),
                        verticalArrangement = Arrangement.SpaceEvenly
                    ) {

                        OutlinedButton(onClick = {
                            startActivity(
                                Intent(
                                    this@MainActivity,
                                    NotificationActivity::class.java
                                )
                            )
                        }, modifier = Modifier.fillMaxWidth()) {
                            Text(text = "我的通知")
                        }

                        OutlinedButton(onClick = {
                            startActivity(Intent(this@MainActivity, DepartmentActivity::class.java))
                        }, modifier = Modifier.fillMaxWidth()) {
                            Text(text = stringResource(id = R.string.department_management))
                        }
                        OutlinedButton(onClick = {
                            startActivity(
                                Intent(
                                    this@MainActivity,
                                    MenuActivity::class.java
                                ).apply {
                                    putExtra(MenuType::name.name, MenuType.Association)
                                })
                        }, modifier = Modifier.fillMaxWidth()) {
                            Text(text = stringResource(id = R.string.association_management))
                        }
                        OutlinedButton(onClick = {
                            startActivity(
                                Intent(
                                    this@MainActivity,
                                    MenuActivity::class.java
                                ).apply {
                                    putExtra(MenuType::name.name, MenuType.Act)
                                })
                        }, modifier = Modifier.fillMaxWidth()) {
                            Text(text = stringResource(id = R.string.activity_management))
                        }

                        OutlinedButton(onClick = {
                            loginViewModel.logout(this@MainActivity) {
                                scaffoldModel.update(message = it)
                            }
                        }, modifier = Modifier.fillMaxWidth()) {
                            Text(text = "退出登录")
                        }
                    }
                }
            }
        }
    }
}