package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.runtime.Composable
import androidx.compose.runtime.ExperimentalComposeApi
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import com.google.accompanist.insets.ExperimentalAnimatedInsets
import com.gyf.csams.R
import com.gyf.csams.main.model.ManagerActViewModel
import com.gyf.csams.module.MAX_SCORE
import com.gyf.csams.module.ManagerActVo
import com.gyf.csams.uikit.RowItem
import com.gyf.csams.uikit.TestTable
import com.gyf.lib.util.DateTimeUtil.datetimeFormat
import java.util.*


/**
 * 查看活动信息
 *
 */
class ManagerActActivity : ComponentActivity() {
    @ExperimentalAnimatedInsets
    @ExperimentalComposeApi
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            TestTable(clazz = ManagerActViewModel::class.java) {
                ActivityTable(vo = it)
            }
        }
    }

    @Composable
    private fun ActivityTable(vo: ManagerActVo) {
        val baseHeight = 50.dp
        RowItem(
            modifier = Modifier.height(baseHeight),
            key = R.string.activity_name, value = vo.activityVo.activityName
        )
        RowItem(
            modifier = Modifier.height(baseHeight),
            key = R.string.activity_association, value = vo.association.name
        )
        RowItem(modifier = Modifier.height(baseHeight), key = R.string.activity_evaluate) {
            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.Center
            ) {
                repeat(MAX_SCORE.toInt()) {
                    Image(
                        modifier = Modifier.weight(1F / MAX_SCORE),
                        painter = painterResource(id = if (it < vo.score) R.drawable.ic_collection_fill else R.drawable.ic_collection),
                        contentDescription = null
                    )
                }
            }
        }
        RowItem(
            modifier = Modifier.height(baseHeight),
            key = R.string.activity_time, value = Date(vo.activityVo.activityTime).datetimeFormat()
        )
        RowItem(
            modifier = Modifier.height(baseHeight * 2),
            key = R.string.activity_location, value = vo.activityVo.activityAddress
        )
    }
}