package com.gyf.csams.main.ui

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.material.Card
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import com.gyf.csams.main.model.MenuType
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainBoxFrame

/**
 * 菜单
 *
 */
class MenuActivity : ComponentActivity() {

    lateinit var menuType: MenuType

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        menuType = intent?.getSerializableExtra(MenuType::name.name) as MenuType

        setContent {
            Body {
                MainBoxFrame(background = { /*TODO*/ }, contentAlignment = Alignment.Center) {
                    Column(
                        modifier = Modifier
                            .fillMaxSize()
                            .border(width = 1.dp, color = MaterialTheme.colors.onBackground)
                            .padding(10.dp), verticalArrangement = Arrangement.SpaceAround
                    ) {
                        menuType.clazz.entries.chunked(2).forEach {
                            Row(
                                modifier = Modifier.fillMaxWidth(),
                                horizontalArrangement = Arrangement.SpaceEvenly
                            ) {
                                MenuItem(menuName = it[0].key, clazz = it[0].value)
                                if (it.size == 2) {
                                    MenuItem(menuName = it[1].key, clazz = it[1].value)
                                } else {
                                    Spacer(modifier = Modifier.size(100.dp))
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @Composable
    private fun MenuItem(menuName: String, clazz: Class<out Activity>) {
        Card(
            modifier = Modifier
                .size(100.dp)
                .clickable(onClick = {
                    startActivity(Intent(this, clazz))
                }),
            backgroundColor = MaterialTheme.colors.background
        ) {
            Box(modifier = Modifier.fillMaxSize(), contentAlignment = Alignment.Center) {
                Text(text = menuName)
            }

        }
    }
}