package com.gyf.csams.main.ui

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material.Card
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.main.model.BackgroundViewModel
import com.gyf.csams.module.NotificationVo
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainColumnFrame
import com.gyf.lib.util.DateTimeUtil.datetimeFormat

import java.util.*

/**
 * 通知
 *
 */
class NotificationActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            Body {
                val model: BackgroundViewModel = viewModel()
                val data by model.data.observeAsState()
                MainColumnFrame(background = { /*TODO*/ }) {
                    if (data?.size == 0) {
                        Row(
                            modifier = Modifier.fillMaxWidth(),
                            horizontalArrangement = Arrangement.Center
                        ) {
                            Text(text = "目前没有收到任何通知")
                        }
                    }
                    LazyColumn {
                        data?.forEach {
                            item {
                                MessageItem(content = it)
                            }
                        }
                    }
                }
            }
        }
    }

    @Composable
    private fun MessageItem(modifier: Modifier = Modifier, content: NotificationVo) {
        Card(modifier = modifier, backgroundColor = MaterialTheme.colors.background) {
            Column(modifier = Modifier.padding(10.dp)) {
                Text(text = content.title, style = MaterialTheme.typography.h5)
                Spacer(modifier = Modifier.height(5.dp))

                Card(
                    backgroundColor = MaterialTheme.colors.background,
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(10.dp)
                ) {
                    Text(text = content.content)
                }

                Spacer(modifier = Modifier.height(10.dp))
                Row(modifier = Modifier.fillMaxWidth(), horizontalArrangement = Arrangement.End) {
                    Text(text = Date(content.createTime).datetimeFormat())
                }
            }
        }
    }
}