package com.gyf.csams.uikit

import androidx.annotation.StringRes
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyListScope
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.platform.LocalDensity
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.insets.*
import com.gyf.lib.model.ScrollViewModel
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainColumnFrame


/**
 * 表格
 *
 * @param callback
 */
@ExperimentalComposeApi
@Composable
fun <A> TestTable(
    clazz: Class<out ScrollViewModel<A>>,
    @StringRes title: Int? = null,
    callback: @Composable (vo: A) -> Unit
) {
    Body {
        MainColumnFrame(background = { /*TODO*/ }) {
            val listState = rememberLazyListState()
            val model = viewModel(modelClass = clazz)
            val data by model.data.observeAsState()
            title?.let {
                Row(
                    horizontalArrangement = Arrangement.Center,
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(10.dp)
                ) {
                    Text(
                        text = stringResource(id = title),
                        style = MaterialTheme.typography.h4
                    )
                }
            }

            LazyColumn(state = listState) {
                data?.forEach {
                    item {
                        callback(vo = it)
                        Spacer(modifier = Modifier.height(10.dp))
                    }
                }

            }
        }
    }
}

@ExperimentalAnimatedInsets
@ExperimentalMaterialApi
@Composable
fun TestTableImeSimple(
    modifier: Modifier = Modifier,
    @StringRes title: Int? = null,
    content: LazyListScope.() -> Unit
) {
    Column(modifier = modifier) {
        title?.let {
            Spacer(modifier = Modifier.height(20.dp))
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .border(width = 1.dp, color = MaterialTheme.colors.onBackground),
                horizontalArrangement = Arrangement.Center,
            ) {
                Text(
                    text = stringResource(id = title),
                    style = MaterialTheme.typography.h4
                )
            }
        }
        val listState = rememberLazyListState()

        val insets = LocalWindowInsets.current
        //TODO 解决键盘遮挡问题
        val isVisible = with(LocalDensity.current) { insets.ime.isVisible }
        val bottom = with(LocalDensity.current) { insets.ime.bottom.toDp() }
        LazyColumn(
            modifier = Modifier
                .fillMaxSize()
                .nestedScroll(connection = rememberImeNestedScrollConnection()),
            contentPadding = PaddingValues(vertical = 10.dp),
            state = listState,
            content = content
        )
    }
}

/**
 * 表格行
 *
 * @param modifier
 * @param key
 * @param value
 * @param content
 */
/**
 * 表格行
 *
 * @param modifier
 * @param key
 * @param value
 * @param content
 */
@Composable
fun RowItem(
    modifier: Modifier = Modifier,
    @StringRes key: Int,
    value: String? = null,
    content: (@Composable () -> Unit)? = null
) {
    Row(
        modifier = modifier
            .fillMaxWidth()
            .border(width = 1.dp, color = Color.Black)
    ) {
        Cell(
            modifier = Modifier.weight(0.5F),
            value = stringResource(id = key)
        )
        Cell(modifier = Modifier.weight(0.5F), value = value, content = content)
    }
}

@Composable
private fun Cell(
    modifier: Modifier = Modifier,
    value: String? = null,
    content: (@Composable () -> Unit)? = null
) {
    Box(
        modifier = modifier
            .fillMaxSize()
            .border(width = 1.dp, color = MaterialTheme.colors.onBackground),
        contentAlignment = Alignment.Center
    ) {
        if (content != null) content() else Text(
            text = value ?: throw IllegalArgumentException(
                "参数错误"
            )
        )
    }
}