package com.gyf.csams.association.model

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.ApiResponse
import com.gyf.csams.module.AuditJoinVo
import com.gyf.csams.module.JoinAssociationVo
import com.gyf.csams.module.SearchExamVo
import com.gyf.lib.model.ScrollViewModel
import com.gyf.lib.util.*
import kotlinx.coroutines.launch

class AuditJoinViewModel(application: Application) :
    ScrollViewModel<JoinAssociationVo>(application = application) {

    fun load(associationId: Int) {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AssociationApi.CheckJoin),
                callback = HttpCallback<MutableList<JoinAssociationVo>>(
                    action = "查看入团申请记录",
                    onSuccess = {
                        it.body?.let {
                            _data.postValue(it)
                        }
                    },
                    typeToken = object :
                        TypeToken<ApiResponse<MutableList<JoinAssociationVo>>>() {}.type
                ), jsonParam =
                SearchExamVo(associationId = associationId, token = TokenManager.getToken())
            )
        }
    }

    /**
     * 审核结果
     *
     * @param joinId
     * @param result
     */
    fun audit(joinId: Int, result: Boolean, callback: (message: String) -> Unit) {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AssociationApi.AuditJoin),
                callback = HttpCallback<Boolean>(action = "审核入团申请", onSuccess = {
                    callback(it.message)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type), jsonParam =
                AuditJoinVo(joinId = joinId, result = result, token = TokenManager.getToken())
            )
        }
    }
}