package com.gyf.csams.association.model

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.*
import com.gyf.lib.uikit.AsyncStringForm
import com.gyf.lib.util.*
import kotlinx.coroutines.launch

/**
 * 社团重命名状态管理
 *
 */
class RenameViewModel : ViewModel() {
    val menuName = "换名申请表"

    val oldName = AsyncStringForm(formDesc = "社团原名", textLength = 10)

    val newName = AsyncStringForm(formDesc = "社团新名", textLength = 10)

    val cause = AsyncStringForm(formDesc = "换名原因", textLength = 30)

    private val _actionResult = MutableLiveData<String>()
    val actionResult: LiveData<String> = _actionResult

    private fun clean() {
        newName.clean()
        cause.clean()
    }

    private val _checkInfo = MutableLiveData<ApiResponse<RenameCheckVo>>()
    val checkInfo: LiveData<ApiResponse<RenameCheckVo>> = _checkInfo

    init {
        read()
    }

    /**
     * 提交重命名申请
     *
     */
    fun apply(associationId: Int) {
        viewModelScope.launch {
            HttpClient.post(
                url = Api.buildUrl(AssociationApi.RenameRegister),
                callback = HttpCallback<Boolean>(action = "提交换名申请表", onSuccess = {
                    it.body?.let {
                        if (it) {
                            clean()
                        }
                    }
                    _actionResult.postValue(it.message)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type),
                jsonParam = RenameApplyVo(

                    rename = RenameVo(
                        newName = newName.getValue(),
                        cause = cause.getValue(),
                        renameId = _checkInfo.value?.body?.renameVo?.renameId
                    ),
                    associationId = associationId,
                    token = TokenManager.getToken()
                )
            )
        }
    }

    /**
     * 查看审核进度
     *
     * @param callback
     */
    private fun read() {
        viewModelScope.launch {
            HttpClient.post(url = Api.buildUrl(AssociationApi.RenameRead),
                callback = HttpCallback<RenameCheckVo>(action = "查看审核进度",
                    onSuccess = { it ->
                        _checkInfo.postValue(it)
                        it.body?.renameVo?.let {
                            newName.setValue(it.newName)
                            cause.setValue(it.cause)
                        }

                    }, typeToken = object : TypeToken<ApiResponse<RenameCheckVo>>() {}.type
                ),
                jsonParam = OnlyToken(clientType = ClientType.Foreground)
            )
        }
    }


}