package com.gyf.csams.association.ui

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.Image
import androidx.compose.foundation.border
import androidx.compose.foundation.layout.*
import androidx.compose.material.IconButton
import androidx.compose.material.MaterialTheme
import androidx.compose.material.OutlinedButton
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.coil.rememberCoilPainter
import com.gyf.csams.R
import com.gyf.csams.association.model.RegAssociationViewModel
import com.gyf.csams.module.CheckStatus
import com.gyf.csams.uikit.Background
import com.gyf.csams.uikit.BackgroundImage
import com.gyf.csams.uikit.CheckTip
import com.gyf.lib.uikit.*
import com.gyf.lib.util.BottomButton
import com.orhanobut.logger.Logger


/**
 * 注册社团
 *
 */
class RegAssociationActivity : ComponentActivity() {

    private val choosePhoto: ChoosePhoto = ChoosePhoto(activity = this)

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContent {
            val model: RegAssociationViewModel = viewModel()
            val checkInfo by model.checkInfo.observeAsState()
            checkInfo?.body.let { it ->
                if (it?.auditCheckVo?.checkStatus == CheckStatus.Finish && it.auditCheckVo.result) {
                    setResult(RESULT_OK, Intent().apply {
                        putExtra(RegAssociationActivity::class.java.name, CheckStatus.Finish.name)
                    })
                    onBackPressed()
                } else {
                    Body {
                        MainColumnFrame(background = {
                            Background(
                                BackgroundImage.RegAssociation,
                                alpha = 0.5F
                            )
                        }) {
                            Spacer(
                                modifier = Modifier
                                    .weight(0.1F)
                            )
                            Title()
                            it?.auditCheckVo?.let { CheckTip(it) }
                            Name()
                            Desc(
                                modifier = Modifier
                                    .weight(0.1F)
                                    .fillMaxWidth()
                            )
                            Spacer(modifier = Modifier.weight(0.05F))
                            Logo(
                                modifier = Modifier
                                    .weight(0.2F)
                                    .fillMaxWidth()
                            )
                            Spacer(modifier = Modifier.weight(0.05F))


                            val scaffoldModel: ScaffoldModel = viewModel()
                            val name by model.name.statusForm.observeAsState()
                            val desc by model.name.statusForm.observeAsState()
                            val fileId by model.fileId.observeAsState()

                            BottomButton(
                                modifier = Modifier.fillMaxWidth(),
                                enabled = name == FormStatus.Valid && desc == FormStatus.Valid && fileId != null,
                                confirmDesc = if (it != null) R.string.reg_again_btn else R.string.reg_btn,

                                ) {
                                model.register {
                                    scaffoldModel.update(message = it, actionLabel = "返回") {
                                        finish()
                                    }
                                }
                            }
                            Spacer(modifier = Modifier.weight(0.05F))
                        }
                    }
                }
            }


        }
    }


    /**
     * 社团Logo
     *
     * @param modifier
     */
    @Composable
    private fun Logo(
        model: RegAssociationViewModel = viewModel(),
        scaffoldModel: ScaffoldModel = viewModel(),
        modifier: Modifier
    ) {

        val uri: Uri? by model.picture.observeAsState()

        val photoLauncher = choosePhoto.photoLauncher(model = model)
        val permissionLauncher = choosePhoto.permissionLauncher(photoLauncher = photoLauncher)


        Box(contentAlignment = Alignment.Center, modifier = modifier) {
            Row(
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.Center, modifier = Modifier
                    .fillMaxSize()
                    .border(width = 1.dp, color = MaterialTheme.colors.onBackground)
            ) {

                if (uri == null) {
                    OutlinedButton(onClick = {
                        choosePhoto.requestPhoto(
                            photoLauncher = photoLauncher,
                            permissionLauncher = permissionLauncher
                        )
                    }) {
                        Text(text = model.picturePlaceHolder)
                    }
                } else {
                    uri.let { it ->
                        if (it != null) {
                            Row {
                                val logo = rememberCoilPainter(request = uri)
                                Image(
                                    painter = logo, contentDescription = null
                                )
                                if (!isReadOnly(model = model)) {
                                    Column(
                                        modifier = Modifier.fillMaxHeight(),
                                        verticalArrangement = Arrangement.SpaceEvenly
                                    ) {
                                        IconButton(onClick = {
                                            choosePhoto.loadPicture(photoLauncher = photoLauncher)
                                        }) {
                                            Image(
                                                painter = painterResource(id = R.drawable.ic_exchange_rate),
                                                contentDescription = null
                                            )
                                        }
                                        IconButton(onClick = {
                                            scaffoldModel.update("确认上传此图片？", actionLabel = "确定") {
                                                Logger.i("开始上传")
                                                model.uploadPhoto {
                                                    scaffoldModel.update(
                                                        message = it,
                                                        actionLabel = "关闭提示"
                                                    )
                                                }
                                            }
                                        }) {
                                            Image(
                                                painter = painterResource(id = R.drawable.ic_upload),
                                                contentDescription = null
                                            )
                                        }
                                    }
                                }
                            }
                        } else {
                            Text(text = model.errorPicture)
                        }
                    }
                }


            }

        }


    }

    /**
     * 菜单标题
     *
     */
    @Composable
    private fun Title(model: RegAssociationViewModel = viewModel()) {
        Row(modifier = Modifier.fillMaxWidth(), horizontalArrangement = Arrangement.Center) {
            Text(text = model.frameDesc, style = MaterialTheme.typography.h4)
        }
    }

    @Composable
    private fun isReadOnly(model: RegAssociationViewModel): Boolean {
        val checkInfo by model.checkInfo.observeAsState()
        val flag = (checkInfo?.body?.let { it.auditCheckVo.checkStatus != CheckStatus.Finish })
        Logger.d("flag=${flag}")
        return flag == true
    }

    /**
     * 社团名称
     * @param model
     */
    @Composable
    private fun Name(model: RegAssociationViewModel = viewModel()) {
        BaseTextField(
            form = model.name, singeLine = true, modifier = Modifier.fillMaxWidth(),
            readOnly = isReadOnly(model = model)
        )
    }

    /**
     * 社团简介
     * @param model
     */
    @Composable
    private fun Desc(model: RegAssociationViewModel = viewModel(), modifier: Modifier) {
        BaseTextField(
            form = model.desc, modifier = modifier,
            readOnly = isReadOnly(model = model)
        )
    }

}

