package com.gyf.csams.message.ui

import android.content.Intent
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.material.Divider
import androidx.compose.material.Icon
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.R
import com.gyf.csams.message.model.MessageType
import com.gyf.csams.message.model.MessageViewModel
import com.gyf.csams.uikit.Background
import com.gyf.csams.uikit.BackgroundImage
import com.gyf.csams.uikit.TextTopAppBar
import com.gyf.lib.uikit.Body
import com.gyf.lib.uikit.MainColumnFrame

/**
 * 消息界面
 *
 */
class MessageActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContent {
            Body {
                val model: MessageViewModel = viewModel()
                MainColumnFrame(background = {
                    Background(
                        image = BackgroundImage.ActivityMessage,
                        alpha = 0.6F
                    )
                }) {
                    TextTopAppBar(title = model.message)
                    Spacer(modifier = Modifier.height(5.dp))
                    Row(
                        modifier = Modifier.fillMaxWidth(),
                        horizontalArrangement = Arrangement.Center
                    ) {
                        Column(
                            horizontalAlignment = Alignment.CenterHorizontally,
                            modifier = Modifier.clickable(onClick = {
                                startActivity(
                                    Intent(
                                        this@MessageActivity,
                                        SysMessageActivity::class.java
                                    )
                                )
                            })
                        ) {
                            Icon(
                                painter = painterResource(id = R.drawable.ic_notice),
                                contentDescription = null,
                                modifier = Modifier.size(30.dp)
                            )
                            Text(text = MessageType.System.desc)
                        }
                    }
                    Spacer(modifier = Modifier.height(5.dp))
                    Divider(
                        modifier = Modifier.height(10.dp),
                        color = MaterialTheme.colors.background
                    )

                }


            }
        }
    }


}