package com.gyf.lib.model

import android.app.Application
import android.net.Uri
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.ApiResponse
import com.gyf.lib.util.*
import kotlinx.coroutines.launch
import java.io.File
import java.io.InputStream

open class ChoosePhotoViewModel(application: Application) : AndroidViewModel(application) {
    protected val _picture = MutableLiveData<Uri?>()
    val picture: LiveData<Uri?> = _picture

    fun setPicture(uri: Uri) {
        _picture.value = uri
    }


    protected fun getInputSteam(): InputStream? {
        _picture.value?.let {
            val resolver = getApplication<Application>().contentResolver
            return resolver.openInputStream(it)
        }
        throw IllegalArgumentException("图片uri为空")
    }

    /**
     * 上传头像
     *
     * @param callback
     */
    fun upload(callback: (flag: Boolean) -> Unit) {
        viewModelScope.launch {
            getInputSteam()?.readBytes()?.apply {
                val context = getApplication<Application>()
                runCatching {
                    val cacheFile = File(context.cacheDir, "${System.currentTimeMillis()}")
                    cacheFile.writeBytes(this@apply)
                    HttpClient.uploadFile(
                        url = Api.buildUrl(AccountApi.UploadHeadimg),
                        HttpCallback<Boolean>(action = "上传头像", onSuccess = {
                            it.body?.let(callback)
                            _picture.postValue(null)
                        }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type),
                        fileList = arrayOf(cacheFile)
                    )
                }
            }
        }
    }
}