package com.gyf.lib.model

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.ApiResponse
import com.gyf.csams.module.OwnInfoVo
import com.gyf.csams.module.Token
import com.gyf.lib.util.*
import com.orhanobut.logger.Logger
import kotlinx.coroutines.launch
import java.lang.reflect.Type

class InitViewModel(application: Application) : AndroidViewModel(application) {
    /**
     * 服务器网络状态是否正常，true=正常，false=不正常
     */
    private val _isNetWorkWorking = MutableLiveData<Boolean>()
    val isNetWorkWorking: LiveData<Boolean> = _isNetWorkWorking

    init {
        checkServer()
    }

    private fun checkServer() {
        Logger.i("测试连接到服务端")
        viewModelScope.launch {
            HttpClient.get(
                Api.buildUrl(TestApi.Test),
                HttpCallback<Boolean>(action = "测试", onSuccess = {
                    _isNetWorkWorking.postValue(true)
                }, onFail = {
                    Logger.e(it)
                    _isNetWorkWorking.postValue(false)
                }, typeToken = object : TypeToken<ApiResponse<Boolean>>() {}.type)
            )
        }

    }

    /**
     * 查询本地是否有且只有一个用户token，如果有则自动登录
     */
    fun <T : OwnInfoVo> hasOnlyUserToken(
        onSuccess: () -> Unit, onFail: () -> Unit, api: AccountApi,
        typeToken: Type
    ) {
        viewModelScope.launch {
            val context = getApplication<Application>()
            val db = AppDatabase.getInstance(context)
            val tokenList = db?.tokenDao()?.queryAll()
            if (tokenList != null && tokenList.size == 1) {
                val currentClientToken: Token = tokenList[0]
                val url = Api.buildUrl(api)
                val action = "校验token"
                Logger.i("${action}api=$url")
                HttpClient.post(
                    url,
                    HttpCallback<T>(
                        action = action,
                        onSuccess = { it ->
                            it.body?.let {
                                Logger.i("token校验结果：${it}")
                                TokenManager.update(it)
                                onSuccess()
                            }

                        },
                        onFail = {
                            Logger.e(it)
                            onFail()
                        }, typeToken = typeToken),
                    jsonParam = currentClientToken
                )

            } else if (tokenList != null && tokenList.size > 1) {

                //TODO 实现切换历史登录帐号
                Logger.i("本地存储了${tokenList.size}个令牌，需要手动登录")
                db.tokenDao().deleteAll()
                Logger.i("清空所有token")
                onFail()
            } else {
                Logger.i("本地没有任何token，跳转到登录界面")
                onFail()
            }
        }
    }

}