package com.gyf.lib.model

import android.app.Application
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.google.gson.reflect.TypeToken
import com.gyf.csams.module.*
import com.gyf.lib.uikit.AsyncStringForm
import com.gyf.lib.util.*
import kotlinx.coroutines.launch

class RefreshViewModel(application: Application) : AndroidViewModel(application) {

    private val _refresh = MutableLiveData<Boolean>()
    val refresh: LiveData<Boolean> = _refresh

    val name = AsyncStringForm(formDesc = "姓名", textLength = 4)
    val desc = AsyncStringForm(formDesc = "简介", textLength = 20)

    fun <T : OwnInfoVo> onSuccess(res: ApiResponse<T>) {
        res.body.let {
            if (it != null) {
                TokenManager.update(it)
            }
            _refresh.postValue(it != null)
        }
    }

    fun refresh() {
        viewModelScope.launch {
            val jsonParam = InfoUpdateVo(
                name = name.formValue.value, desc = desc.formValue.value,
                token = TokenManager.getToken(),
                clientType = TokenManager.getClientType()
            )
            when (jsonParam.clientType) {
                ClientType.Foreground -> HttpClient.post(
                    url = Api.buildUrl(AccountApi.Refresh),
                    callback = HttpCallback<UserVo>(
                        action = "刷新个人信息",
                        onSuccess = { onSuccess(it) },
                        typeToken = object : TypeToken<ApiResponse<UserVo>>() {}.type
                    ),
                    jsonParam = jsonParam
                )
                ClientType.Background -> HttpClient.post(
                    url = Api.buildUrl(AccountApi.Refresh),
                    callback = HttpCallback<ManagerVo>(
                        action = "刷新个人信息",
                        onSuccess = { onSuccess(it) },
                        typeToken = object : TypeToken<ApiResponse<ManagerVo>>() {}.type
                    ),
                    jsonParam = jsonParam
                )
            }

        }
    }
}