package com.gyf.lib.uikit

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.csams.module.OwnInfoVo
import com.gyf.lib.BuildConfig
import com.gyf.lib.model.InitViewModel
import com.gyf.lib.util.AccountApi

import com.orhanobut.logger.Logger
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import java.lang.reflect.Type

abstract class AbstractInitActivity<T : OwnInfoVo> : ComponentActivity() {


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)


        setContent {
            Body {
                MainBoxFrame(background = { /*TODO*/ }, contentAlignment = Alignment.Center) {
                    val initViewModel: InitViewModel = viewModel()
                    //检查网络
                    val isNetWorkWorking: Boolean? by initViewModel.isNetWorkWorking.observeAsState(
                        null
                    )
                    Logger.i("初始化")
                    when (isNetWorkWorking) {
                        null -> AnimationText(text = "测试服务端运行状态中。。。")
                        true -> init(initViewModel = initViewModel)
                        false -> AnimationText(text = "无法连接到服务端，请检查服务端地址${BuildConfig.SERVER_ADDRESS}是否配置正确")
                    }
                }
            }
        }
    }

    protected abstract val main: Class<out Activity>

    protected abstract val login: Class<out Activity>

    abstract val api: AccountApi

    abstract val typeToken: Type

    private fun init(initViewModel: InitViewModel) {
        //后台检查token
        initViewModel.hasOnlyUserToken<T>(onSuccess = {
            startActivity(Intent(this, main))
        }, onFail = {
            startActivity(Intent(this, login))
        }, api = api, typeToken = typeToken)
        GlobalScope.launch {
            delay(1000)
            finish()
        }
    }

}

