package com.gyf.lib.uikit

import android.Manifest
import android.content.Intent
import android.content.pm.PackageManager
import android.provider.MediaStore
import androidx.activity.ComponentActivity
import androidx.activity.compose.ManagedActivityResultLauncher
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.result.ActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.runtime.Composable
import androidx.core.content.ContextCompat
import androidx.lifecycle.viewmodel.compose.viewModel
import com.gyf.lib.R
import com.gyf.lib.model.ChoosePhotoViewModel
import com.orhanobut.logger.Logger

class ChoosePhoto(val activity: ComponentActivity) {

    fun loadPicture(photoLauncher: ManagedActivityResultLauncher<Intent, *>) {
        photoLauncher.launch(
            Intent(
                Intent.ACTION_PICK,
                MediaStore.Images.Media.EXTERNAL_CONTENT_URI
            )
        )
    }

    @Composable
    fun photoLauncher(model: ChoosePhotoViewModel = viewModel()): ManagedActivityResultLauncher<Intent, ActivityResult> {
        return rememberLauncherForActivityResult(contract = ActivityResultContracts.StartActivityForResult()) {
            when (it.resultCode) {
                ComponentActivity.RESULT_OK -> {

                    it.data?.data?.apply {
                        Logger.i("uri=$this")
                        model.setPicture(this)
                    }
                }
            }
        }
    }

    @Composable
    fun permissionLauncher(
        photoLauncher: ManagedActivityResultLauncher<Intent, *>,
        scaffoldModel: ScaffoldModel = viewModel()
    ): ManagedActivityResultLauncher<String, Boolean> {

        return rememberLauncherForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                // Permission Accepted: Do something
                loadPicture(photoLauncher = photoLauncher)
            } else {
                // Permission Denied: Do something
                scaffoldModel.update(message = activity.getString(R.string.denined_photo_permission))
            }
        }
    }

    fun requestPhoto(
        photoLauncher: ManagedActivityResultLauncher<Intent, *>,
        permissionLauncher: ManagedActivityResultLauncher<String, Boolean>
    ) {
        when (PackageManager.PERMISSION_GRANTED) {
            ContextCompat.checkSelfPermission(
                activity.applicationContext,
                Manifest.permission.READ_EXTERNAL_STORAGE
            ) -> {
                // Some works that require permission
                loadPicture(photoLauncher = photoLauncher)
            }
            else -> {
                // Asking for permission
                permissionLauncher.launch(Manifest.permission.READ_EXTERNAL_STORAGE)
            }
        }
    }

}
