package com.gyf.lib.uikit

import androidx.compose.foundation.Image
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.runtime.*
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import com.google.accompanist.coil.rememberCoilPainter
import com.google.accompanist.imageloading.ImageLoadState
import com.gyf.csams.module.ManagerInfoVo
import com.gyf.csams.module.ManagerVo
import com.gyf.csams.module.PersonInfoVo
import com.gyf.csams.module.UserVo
import com.gyf.lib.model.ChoosePhotoViewModel
import com.gyf.lib.model.RefreshViewModel
import com.gyf.lib.util.Api
import com.orhanobut.logger.Logger


@Composable
fun Profile(
    modifier: Modifier,
    personInfoVo: PersonInfoVo,
    setHeadImg: () -> Unit = {},
    choosePhotoViewModel: ChoosePhotoViewModel = viewModel(),
    refreshViewModel: RefreshViewModel = viewModel(),
    readOnly: Boolean = false,
    duty: @Composable () -> Unit = {
        Text(
            text = when (personInfoVo) {
                is UserVo -> personInfoVo.manager?.duty?.desc ?: "----"
                is ManagerVo -> personInfoVo.duty.desc
                is ManagerInfoVo -> personInfoVo.duty.desc
                else -> "-----"
            }
        )
    }
) {

    Column(
        modifier = modifier.border(1.dp, Color.Black),
        verticalArrangement = Arrangement.SpaceEvenly
    ) {
        var isEdit by remember {
            mutableStateOf(false)
        }
        Logger.i("info:${personInfoVo}")
        Row(
            modifier = Modifier
                .fillMaxWidth()
                .weight(0.7F)
                .border(width = 1.dp, color = MaterialTheme.colors.onBackground),
            horizontalArrangement = Arrangement.SpaceBetween
        ) {
            val picture by choosePhotoViewModel.picture.observeAsState()
            picture?.let {
                Logger.i("picture=${it}")
                choosePhotoViewModel.upload {
                    refreshViewModel.refresh()
                }
            }
            personInfoVo.headImg.let {
                when {
                    it != null -> {
                        val headImg = rememberCoilPainter(request = Api.buildUrl(it))

                        when (headImg.loadState) {
                            is ImageLoadState.Loading ->
                                Box(
                                    modifier = Modifier
                                        .weight(0.4F)
                                        .fillMaxHeight(), contentAlignment = Alignment.Center
                                ) {
                                    CircularProgressIndicator()
                                }
                            is ImageLoadState.Success -> Image(
                                painter = headImg,
                                contentDescription = null,
                                modifier = Modifier
                                    .weight(0.4F)
                                    .fillMaxHeight()
                                    .clickable(onClick = setHeadImg)
                            )
                            else -> Text(text = "图片加载失败")
                        }
                    }
                    !readOnly -> Box(
                        modifier = Modifier
                            .weight(0.4F)
                            .fillMaxHeight(),
                        contentAlignment = Alignment.Center
                    ) {
                        OutlinedButton(onClick = setHeadImg) {
                            Text(text = "设置头像")
                        }
                    }
                }
            }

            Column(
                modifier = Modifier
                    .weight(0.4F)
                    .fillMaxHeight()
                    .border(width = 1.dp, color = MaterialTheme.colors.onBackground),
                verticalArrangement = Arrangement.SpaceEvenly,
                horizontalAlignment = Alignment.CenterHorizontally
            ) {
                if (isEdit) {
                    BaseTextField(form = refreshViewModel.name)
                } else {
                    Text(text = personInfoVo.name)
                }
                duty()
            }
        }


        Card(
            backgroundColor = MaterialTheme.colors.background,
            modifier = Modifier.weight(0.3F)
        ) {

            Row(
                modifier = Modifier.fillMaxWidth()
            ) {
                Box(
                    contentAlignment = Alignment.Center, modifier = Modifier
                        .weight(0.8F)
                        .fillMaxHeight()
                ) {
                    if (isEdit) {
                        BaseTextField(form = refreshViewModel.desc)
                    } else {
                        Text(text = personInfoVo.desc)
                    }
                }
                if (!readOnly) {
                    Box(
                        modifier = Modifier
                            .weight(0.2F)
                            .fillMaxHeight(),
                        contentAlignment = Alignment.Center
                    ) {
                        OutlinedButton(onClick = {
                            isEdit = !isEdit
                            if (isEdit) {
                                refreshViewModel.name.setValue(personInfoVo.name)
                                refreshViewModel.desc.setValue(personInfoVo.desc)
                            } else {
                                refreshViewModel.refresh()
                            }
                        }) {
                            if (isEdit) {
                                Text(text = "完成")
                            } else {
                                Text(text = "编辑")
                            }

                        }
                    }
                }

            }


        }
    }
}