package com.gyf.lib.util

import android.app.Activity
import androidx.annotation.StringRes
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.width
import androidx.compose.material.MaterialTheme
import androidx.compose.material.OutlinedButton
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import com.gyf.lib.R

@Composable
fun BottomButton(
    modifier: Modifier = Modifier,
    enabled: Boolean = true,
    @StringRes confirmDesc: Int = R.string.confirm_btn,
    @StringRes backDesc: Int? = R.string.back_btn,
    onBack: (() -> Unit)? = null,
    onConfirm: () -> Unit
) {
    val context = LocalContext.current as? Activity
    Row(modifier = modifier, horizontalArrangement = Arrangement.Center) {
        OutlinedButton(
            onClick = onConfirm,
            enabled = enabled,
            modifier = Modifier.background(color = MaterialTheme.colors.primary)
        ) {
            Text(text = stringResource(id = confirmDesc))
        }

        if (backDesc != null) {
            Spacer(modifier = Modifier.width(10.dp))
            OutlinedButton(onClick = {
                if (onBack == null) context?.onBackPressed() else onBack()
            }, modifier = Modifier.background(color = MaterialTheme.colors.secondary)) {
                Text(text = stringResource(id = backDesc))
            }
        }

    }
}

