/*
 * Copyright (C) 2017 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.gyf.lib.util

import android.app.NotificationChannel
import android.app.NotificationManager
import android.content.Context
import android.os.Build
import androidx.core.app.NotificationCompat
import com.gyf.lib.BuildConfig


/**
 * Simplifies common [Notification] tasks.
 */
object NotificationUtil {
    // The id of the channel.
    const val CHANNEL_ID = BuildConfig.LIBRARY_PACKAGE_NAME

    fun createNotificationChannel(
        context: Context,
        channelImportance: Int
    ): String? {
        // NotificationChannels are required for Notifications on O (API 26) and above.
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {

            // The user-visible name of the channel.
            val channelName: CharSequence = ""
            // The user-visible description of the channel.
            val channelDescription = ""
            val channelEnableVibrate = false
            val channelLockscreenVisibility: Int = NotificationCompat.VISIBILITY_PUBLIC

            // Initializes NotificationChannel.
            val notificationChannel =
                NotificationChannel(CHANNEL_ID, channelName, channelImportance)
            notificationChannel.description = channelDescription
            notificationChannel.enableVibration(channelEnableVibrate)
            notificationChannel.lockscreenVisibility = channelLockscreenVisibility

            // Adds NotificationChannel to system. Attempting to create an existing notification
            // channel with its original values performs no operation, so it's safe to perform the
            // below sequence.
            val notificationManager: NotificationManager =
                context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
            notificationManager.createNotificationChannel(notificationChannel)
            CHANNEL_ID
        } else {
            // Returns null for pre-O (26) devices.
            null
        }
    }
}